<?php

namespace Drupal\cleantaxonomy\Commands;

use Drush\Commands\DrushCommands;
use Drupal\Core\Database\Connection;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Entity\EntityFieldManagerInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * A Drush commandfile.
 *
 * In addition to this file, you need a drush.services.yml
 * in root of your module, and a composer.json file that provides the name
 * of the services file to use.
 *
 * See these files for an example of injecting Drupal services:
 *   - http://cgit.drupalcode.org/devel/tree/src/Commands/DevelCommands.php
 *   - http://cgit.drupalcode.org/devel/tree/drush.services.yml
 */
class CleantaxonomyCommands extends DrushCommands {
  /**
   * The database service.
   *
   * @var \Drupal\Core\Database\Connection
   */
  protected $database;

  /**
   * The entity type manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * The entity type manager.
   *
   * @var \Drupal\Core\Entity\EntityFieldManagerInterface
   */
  protected $entityFieldManager;

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
        $container->get('database'),
        $container->get('entity_type.manager'),
        $container->get('entity_field.manager')
    );
  }

  /**
   * Constructs a cleantaxonomy object.
   *
   * @param \Drupal\Core\Database\Connection $database
   *   A database connection.
   *
   *   The entity type manager.
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   *   Entity type manager.
   *
   *   The entity field manager.
   * @param \Drupal\Core\Entity\EntityFieldManagerInterface $entityFieldManager
   *   The entity field manager.
   */
  public function __construct(Connection $database, EntityTypeManagerInterface $entity_type_manager, EntityFieldManagerInterface $entityFieldManager) {
    $this->database = $database;
    $this->entityTypeManager = $entity_type_manager;
    $this->entityFieldManager = $entityFieldManager;
  }

  /**
   * Replace taxonomy terms using cleantaxonomy.
   *
   * @param string $term1
   *   Name of the old term to be replaced.
   * @param string $term2
   *   Name of the new term.
   *
   * @usage drush ct-replace term1 term2
   *   replaces the term1 with term2.
   *
   * @command cleantaxonomy:replace-callback
   * @aliases ct-replace,cleantaxonomy-replace-callback
   */
  public function replaceCallback($term1, $term2) {
    $name1 = $term1;
    $name2 = $term2;
    if ($name1 === $name2) {
      throw new \Exception("Both terms are same. Please choose different terms.");
    }
    $term_from = $this->database->select('taxonomy_term_field_data', 'x')
      ->fields('x', ['tid']);
    $term_from->condition('x.name', $name1);
    $data = $term_from->execute();
    $term_from = $data->fetchAll(\PDO::FETCH_ASSOC);
    $term_from = $term_from[0]['tid'];

    $term2_obj = $this->entityTypeManager->getStorage('taxonomy_term')->loadByProperties(['name' => $name2]);

    if (empty($term_from) || empty($term2_obj)) {
      throw new \Exception("Terms not found! Please enter correct terms.");
    }
    else {
      $term2_object = reset($term2_obj);
      $term_to = $term2_object->id();

      // Retrieving nids having taxonomy term 'from'.
      $duplicate = $this->database->select('taxonomy_index', 'x')
        ->fields('x', ['nid']);
      $duplicate->condition('x.tid', $term_from);
      $data = $duplicate->execute();
      $nodes_with_term_from = $data->fetchAll(\PDO::FETCH_ASSOC);

      $entityFieldManager = $this->entityFieldManager;
      foreach ($nodes_with_term_from as $value) {
        $node_storage = $this->entityTypeManager->getStorage('node');
        $node = $node_storage->load($value['nid']);
        $bundle = $node->bundle();
        $fields = $entityFieldManager->getFieldDefinitions('node', $bundle);
        foreach ($node as $key => $values) {
          if (substr($key, 0, 5) == 'field') {
            if ($fields[$key]->getSettings()['target_type'] == 'taxonomy_term') {
              $field_machine_name = $fields[$key]->getName();
              $taxonomy_terms = $node->get($field_machine_name)->getValue();
              $updated_terms = [];
              foreach ($taxonomy_terms as $target_id) {
                if ($target_id['target_id'] == $term_from) {
                  $updated_terms[] = ['target_id' => $term_to];
                }
                else {
                  $updated_terms[] = ['target_id' => $target_id['target_id']];
                }
              }
              $updated_terms = array_unique($updated_terms);
              $node->set($field_machine_name, $updated_terms);
              $node->save();
            }
          }
        }
      }
      $this->logger()->success("Replaced Successfully.");
    }
  }

}
