<?php

namespace Drupal\cleantaxonomy\Form;

use Drupal\Core\Database\Connection;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Form\FormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Url;
use Drupal\node\Entity\Node;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Implements an CleanTaxonomyReplaceForm form.
 */
class CleanTaxonomyReplaceForm extends FormBase {


  /**
   * The database service.
   *
   * @var \Drupal\Core\Database\Connection
   */
  protected $database;

  /**
   * The entity type manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * The entity type manager.
   *
   * @var \Drupal\Core\Entity\EntityFieldManagerInterface
   */
  protected $entityFieldManager;

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('database'),
      $container->get('entity_type.manager')
    );
  }

  /**
   * Constructs a cleantaxonomy object.
   *
   * @param \Drupal\Core\Database\Connection $database
   *   A database connection.
   *
   *   The entity type manager.
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   *   Entity type manager.
   */
  public function __construct(Connection $database, EntityTypeManagerInterface $entity_type_manager) {
    $this->database = $database;
    $this->entityTypeManager = $entity_type_manager;
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'cleantaxonomy_replace';
  }

  /**
   * Batch process to replace taxonomy terms.
   */
  public static function replaceTerm($nids, $termto, $termfrom, &$context) {
    $entityFieldManager = \Drupal::service('entity_field.manager');
    $results = [];
    foreach ($nids as $value) {

      $node = Node::load($value);
      $bundle = $node->bundle();

      $fields = $entityFieldManager->getFieldDefinitions('node', $bundle);
      foreach ($node as $key => $values) {
        if (substr($key, 0, 5) == 'field') {
          if ($fields[$key]->getSettings()['target_type'] == 'taxonomy_term') {

            $field_machine_name = $fields[$key]->getName();
            $taxonomy_terms = $node->get($field_machine_name)->getValue();
            $updated_terms = [];
            foreach ($taxonomy_terms as $target_id) {
              if ($target_id['target_id'] == $termfrom) {
                $updated_terms[] = ['target_id' => $termto];
              }
              elseif ($target_id['target_id'] != $termto) {
                $updated_terms[] = ['target_id' => $target_id['target_id']];
              }
            }
            $node->set($field_machine_name, $updated_terms);
            $node->save();
            $nid = $node->id();
            $results[] = $nid;
          }
        }
      }
    }
    $context['results'] = $results;
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state, $tid = NULL) {
    $currentUrl = Url::fromRoute('<current>');
    $path = $currentUrl->getInternalPath();
    $argument = explode('/', $path);
    $id = $argument[2];
    $term_storage = $this->entityTypeManager->getStorage('taxonomy_term');
    $taxonomy_term_object = $term_storage->load($id);
    if (empty($taxonomy_term_object)) {
      $form['error'] = [
        'type' => 'markup',
        '#markup' => $this->t('There is no term exist with tid @id', ['@id' => $id]),
      ];
      return $form;
    }
    $taxonomy_term_name = $taxonomy_term_object->get('name')->value;
    $taxonomy_term_vid = $taxonomy_term_object->bundle();
    $this->messenger()->addWarning($this->t('It will replace the nodes attached to the previous taxonomy term.'));
    $form['cleantaxonomy']['Term1'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Taxonomy Term'),
      '#default_value' => $taxonomy_term_name,
      '#attributes' => ['readonly' => 'readonly'],
      '#settings' => ['target_type' => 'taxonomy_term'],
      '#disabled' => 'disabled',
      '#size' => 60,
      '#maxlength' => 128,
      '#weight' => '5',
    ];

    $form['cleantaxonomy']['Term2'] = [
      '#type' => 'textfield',
      '#title' => $this->t('New Taxonomy Term'),
      '#weight' => '10',
      '#maxlength' => 128,
      '#description' => '',
      '#autocomplete_route_name' => 'cleantaxonomy.autocomplete.taxonomy_entity_type',
      '#autocomplete_route_parameters' => ['taxonomy_term_vid' => $taxonomy_term_vid],
    ];

    $form['cleantaxonomy']['create_new_term'] = [
      '#type' => 'checkbox',
      '#title' => $this->t("Create <b>New Taxonomy Term</b> if it doesn't already exists."),
      '#default_value' => 0,
      '#weight' => '15',
    ];

    $form['taxonomy_term_vid'] = [
      '#type' => 'hidden',
      '#value' => $taxonomy_term_vid,
    ];
    $form['submit'] = [
      '#type' => 'submit',
      '#weight' => '20',
      '#value' => $this->t('Replace Term'),
    ];

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function validateForm(array &$form, FormStateInterface $form_state) {
    $term_to_replace = $form_state->getValue('Term1');
    $new_term = $form_state->getValue('Term2');

    if (empty($new_term)) {
      $form_state->setErrorByName('Term2', $this->t('Please provide a Term Value.'));
    }

    $create_new_term = $form_state->getValue('create_new_term');
    $taxonomy_term_vid = $form_state->getValue('taxonomy_term_vid');
    $term_storage = $this->entityTypeManager->getStorage('taxonomy_term');
    $new_term_object = $this->retrieveTermData($taxonomy_term_vid, $new_term);

    if (!isset($new_term_object)) {
      if ($create_new_term === 0) {
        $form_state->setErrorByName('Term2', $this->t('The "@new_term" Taxonomy Term Does Not Exists in "@taxonomy_term_vid" Taxonomy Vocabulary.',
        [
          '@new_term' => $new_term,
          '@taxonomy_term_vid' => $taxonomy_term_vid,
        ]
        ));
      }
      else {
        $new_term_object = $term_storage->create([
          'vid' => $taxonomy_term_vid,
          'name' => $new_term,
        ]);
        $new_term_object->save();
        $new_term_name = $new_term_object->getName();
      }
    }
    else {
      $new_term_name = $new_term_object->getName();
    }
    if ($term_to_replace == $new_term_name) {
      $form_state->setErrorByName('Term2', $this->t('Please choose a term other than @new_term_name.', ['@new_term_name' => $new_term_name]));
    }
  }

  /**
   * Implements submitForm().
   *
   * The submitForm function replaces one taxonomy term with another for
   * all nodes that are tagged with the first term.
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $taxonomy_term_vid = $form_state->getValue('taxonomy_term_vid');
    $name1 = $form_state->getValue('Term1');
    $term_from = $this->database->select('taxonomy_term_field_data', 'x')
      ->fields('x', ['tid']);
    $term_from->condition('x.name', $name1);
    $data = $term_from->execute();
    $term_from = $data->fetchAll(\PDO::FETCH_ASSOC);
    $term_from = $term_from[0]['tid'];

    $term_to_name = $form_state->getValue('Term2');
    $new_term = $this->retrieveTermData($taxonomy_term_vid, $term_to_name);
    $name2 = $new_term->getName();
    $term_to = $new_term->id();
    // Retrieving nids having taxonomy term 'from'.
    $duplicate = $this->database->select('taxonomy_index', 'x')
      ->fields('x', ['nid']);
    $duplicate->condition('x.tid', $term_from);
    $data = $duplicate->execute();
    $nodes_with_term_from = array_keys($data->fetchAllAssoc('nid'));
    $node_terms = array_chunk($nodes_with_term_from, 100);
    $batch = [
      'title' => $this->t('Replacing terms ...'),
      'operations' => [],
      'init_message'     => $this->t('Initiating'),
      'progress_message' => $this->t('Generated @current out of @total.'),
      'error_message'    => $this->t('An error occurred during processing'),
    ];
    foreach ($node_terms as $node_term) {
      $batch['operations'][] = [
        '\Drupal\cleantaxonomy\Form\CleanTaxonomyReplaceForm::replaceTerm',
        [$node_term, $term_to, $term_from],
      ];
    }
    batch_set($batch);
    $this->messenger()->addMessage($this->t('All the nodes tagged to term %string1 are now tagged to term %string2.',
     [
       '%string1' => $name1,
       '%string2' => $name2,
     ]));
  }

  /**
   * Implements retrieveTermData().
   */
  public function retrieveTermData($taxonomy_term_vid, $new_term) {
    $term_storage = $this->entityTypeManager->getStorage('taxonomy_term');
    $query = $this->database->select('taxonomy_term_field_data', 'ttfd');
    $query->fields('ttfd', ['tid', 'vid', 'name']);
    $query->condition('vid', $taxonomy_term_vid, '=');
    $query->condition('name', $new_term);
    $result = $query->execute()->fetch();
    $new_term_id = ($result != FALSE) ? $result->tid : '';
    $new_term_object = $term_storage->load($new_term_id);
    return $new_term_object;
  }

}
